var order,
    tiles = [],
    grid = {},

    allTilesMoved = false,
    tilesAtDestination,
    tileRatio = 1,
    canvasWidth = window.innerWidth,
    canvasHeight = window.innerHeight,

    tileImagesLoaded = 0,

    context2D = document.getElementById("canvas-container").getContext("2d"),

    thumbnails = { //collage
        image: "img_grid.jpeg",
        sizeThumbX: 70,
        sizeThumbY: 99
    },

    desiredFps = 60,

    lastFrameMillisecs = Date.now(),

    amountOfTiles= 100;


function gup( name, url ) {
    //if (!url) url = location.href;
    name = name.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
    var regexS = "[\\?&]"+name+"=([^&#]*)";
    var regex = new RegExp( regexS );
    var results = regex.exec( url );
    console.log('amount of tiles: ' + results[1]);
    return results == null ? null : results[1];
}

function Tile() {
    this.width = 20;
    this.height = 40;
    this.x = 0;
    this.y = 0;
    this.destX = 0;
    this.destY = 0;
    this.isFilteredOut = false;
    this.thumb = {};
}

Tile.prototype.loadImage = function(src) {
    var that = this,
        tmpImg = new Image();

    tmpImg.onload = function() {
        that.isTextureReady = true;
        that.isThumbLoaded = true;
        that.imageResource = tmpImg;
        tileImagesLoaded++;
    };

    tmpImg.src = src;
};

Tile.prototype.interpolate = function(strength) {
    this.x += (this.destX - this.x) * strength;
    this.y += (this.destY - this.y) * strength;
};


Tile.prototype.render = function() {
    if (this.roundTwoDecimal(this.destX) === this.roundTwoDecimal(this.x) && this.roundTwoDecimal(this.destY) === this.roundTwoDecimal(this.y)) {
        this.isAtDestination = true;
    }

    this.itemsPerRow = this.imageResource.width / this.thumb.w;
    this.spriteX = (this.thumb.pos % this.itemsPerRow) * this.thumb.w;
    this.spriteY = Math.floor(this.thumb.pos / this.itemsPerRow) * this.thumb.h;

    context2D.drawImage(this.imageResource, this.spriteX, this.spriteY, this.thumb.w, this.thumb.h, this.x, this.y, this.width, this.height);

};

Tile.prototype.roundTwoDecimal = function(value) {
    return Math.round(value * 100) / 100;
};


function setupTiles() {

    createTiles(amountOfTiles);
    // tiles is now filled
    order = getOrdinalOrder(tiles.length);

    // order is ready
    var x = 3;
    var y = 4;

    tileRatio = x / y;

    prepareTileGrid();
    resizeCols();

    updateTilePositions();
}

function createTiles(n) {
    var minX = 0,
        maxX = canvasWidth,
        minY = 0,
        maxY = canvasHeight;

    for (var i = 0; i < n; i++) {

        tiles[i] = new Tile();

        tiles[i].thumb = {
            w: thumbnails.sizeThumbX,
            h: thumbnails.sizeThumbY
        };

        tiles[i].loadImage(thumbnails.image);

        tiles[i].x = (Math.random() * ((maxX - minX) + 1) + minX);
        tiles[i].y = (Math.random() * ((maxY - minY) + 1) + minY);

        tiles[i].thumb.pos = i;
    }
}

function updateTilePositions() {
    var orderPos = 0;

    for (var i = 0; i < grid.rows; i++) {
        for (var j = 0; j < grid.cols; j++) {

            if (orderPos >= order.length) {
                break;
            }

            var currentTile = tiles[order[i * grid.cols + j]];
            currentTile.width = grid.tileWidth;
            currentTile.height = grid.tileHeight;

            currentTile.destX = j * grid.tileWidth;
            currentTile.destY = i * grid.tileHeight;
            orderPos++;
        }

    }

}

function getOrdinalOrder(n) {
    var order = [];
    for (var i = 0; i < n; i++) {
        order[i] = i;
    }
    return order;
}

function prepareTileGrid() {

    grid = grid || {};

    grid.cols = Math.ceil((Math.sqrt(tiles.length)) / tileRatio);
    grid.rows = Math.ceil((Math.sqrt(tiles.length)) * tileRatio);

    grid.tileWidth = canvasWidth / grid.cols;
    grid.tileHeight = grid.tileWidth / tileRatio;

}

function resizeCols() {

    var usedRows = tiles.length / grid.cols;
    var targetRows = usedRows + usedRows / grid.cols;
    var targetTileWidth = canvasWidth / (grid.cols);
    var targetTileHeight  = targetTileWidth / tileRatio;

    if (targetRows * targetTileHeight < canvasHeight) {
        decreaseCols();
    } else {
        increaseCols();
    }

}

function increaseCols() {
    var usedRows = tiles.length / grid.cols;
    var targetRows = Math.ceil(usedRows - usedRows / grid.cols);

    var currentTileWidth = canvasWidth / grid.cols;
    var currentTileHeight  = currentTileWidth / tileRatio;

    var targetTileWidth = canvasWidth / (grid.cols + 1);
    var targetTileHeight  = targetTileWidth / tileRatio;

    if (grid.rows * currentTileHeight > canvasHeight) {
        grid.cols++;

        if (targetRows * grid.cols < tiles.length) {
            grid.rows = Math.ceil(targetRows * 0.5 * usedRows);
        } else {
            grid.rows = targetRows;
        }
        grid.tileWidth = targetTileWidth;
        grid.tileHeight = targetTileHeight;
        increaseCols();
    }

}

function decreaseCols() {
    var usedRows = tiles.length / grid.cols;
    var targetRows = Math.ceil(usedRows + usedRows / grid.cols);

    var targetTileWidth = canvasWidth / (grid.cols - 1);
    var targetTileHeight  = targetTileWidth / tileRatio;

    if (targetRows * targetTileHeight < canvasHeight) {
        grid.cols--;
        grid.rows = targetRows;
        grid.tileWidth = targetTileWidth;
        grid.tileHeight = targetTileHeight;
        decreaseCols();
    }
}



function mainloop() {
    // delta timing
    lastFrameMillisecs = Date.now();
    if (!allTilesMoved && tileImagesLoaded === tiles.length) {
        renderFrame();
    }

}

function render() {
    context2D.clearRect(0, 0, canvasWidth, canvasHeight);

    for (var i= 0, len = tiles.length; i < len; i++) {
        tiles[i].interpolate(parseFloat(0.5));
        tiles[i].render();
    }
}

function renderFrame() {
    tilesAtDestination = 0;

    context2D.clearRect(0, 0, canvasWidth, canvasHeight);

    for (var i= 0, len = tiles.length; i < len; i++) {
        if (tiles[i].isAtDestination) {
            tilesAtDestination++;
        }

        tiles[i].interpolate(parseFloat(0.5));
        tiles[i].render();
    }


    if (tilesAtDestination === tiles.length) {
        allTilesMoved = true;
    }

}

var canvasNativeNode = context2D.canvas;
canvasNativeNode.width = canvasWidth;
canvasNativeNode.height = canvasHeight;

setupTiles();

var mainloopIntervalHandler = setInterval(function() {
    mainloop();
}, 1000 / desiredFps);
